#include <cstdlib>
#include <string>
#include <fstream>
#include <sstream>
#include <iomanip>

#include "tool.h"
#include "init.h"
#include "sds_lib.h"

#include <iostream>

void initVectMap(fixed16_2 *vectMap[MAP_SIZE][MAP_SIZE]) {
    for (int i = 0; i < MAP_SIZE; i++) {
        for (int j = 0; j < MAP_SIZE; j++) {
            for (int k = 0; k < VECT_DIM; k++) {
                vectMap[i][j][k] = (fixed16_2)(randf() / VECT_DIM);
            }
        }
    }
}

void initLabelMap(int labelMap[MAP_SIZE][MAP_SIZE]) {
    for (int i = 0; i < MAP_SIZE; i++) {
        for (int j = 0; j < MAP_SIZE; j++) {
            labelMap[i][j] = -1;
        }
    }
}

/// <summary>
/// サンプルデータのデータ数を数える
/// </summary>
/// <param name="filename">ファイル名</param>
int countSampleData(const char *filename) {
    int lines = 0;

    std::ifstream ifs(filename);
    if (!ifs) return -1;

    std::string line;
    while (std::getline(ifs, line))
        lines++;

    return lines;
}

/// <summary>
/// データ読み込み
/// </summary>
/// <param name="filename">ファイル名</param>
/// <param name="data">読み込んだデータ</param>
/// <param name="dataNum">データ数</param>
void inputStraightData(const char *filename, fixed16_2 **data, int dataNum) {
    std::ifstream ifs(filename);

    std::string line, token;
    for (int i = 0; i < dataNum; i++) {
        std::getline(ifs, line);
        std::istringstream stream(line);

        // カンマ区切りで読み込み、浮動小数点数に変換
        for (int k = 0; k < VECT_DIM && getline(stream, token, ','); k++) {
            data[i][k] = atof(token.c_str());
        }
    }
}

void hangSampleData(fixed16_2 **data[2], int dataNum[2]) {
    const char *path[2] = {POSITIVE_PATH, NEGATIVE_PATH};

    for (int c = 0; c < 2; c++) {
        // データ配列・データ番号配列の作成
        dataNum[c] = countSampleData(path[c]);

        // 配列確保
        data[c] = (fixed16_2 **)malloc(sizeof(fixed16_2 *) * dataNum[c]);
        fixed16_2 *ptr = (fixed16_2 *)sds_alloc(sizeof(fixed16_2) * (dataNum[c] * VECT_DIM));
        for (int i = 0; i < dataNum[c]; i++) {
            data[c][i] = &ptr[i * VECT_DIM];
        }

        // データ読み込み
        inputStraightData(path[c], data[c], dataNum[c]);
    }
}
