#include <cstdio>
#include <iomanip>
#include <iostream>

#include "env.h"
#include "init.h"
#include "som.h"
#include "som_learn.h"
#include "tool.h"

#include "sds_time.h"

int main(int argc, char *argv[]) {

    /* 各種データ格納用変数 */
    /// <summary> ベクトル用マップ </summary>
    fixed16_2 *vectMap[MAP_SIZE][MAP_SIZE];
    /// <summary> ラベル用マップ </summary>
    int labelMap[MAP_SIZE][MAP_SIZE];
    /// <summary> 学習データ格納 [分類][ファイル][次元数] </summary>
    fixed16_2 **trainData[2];
    /// <summary> 学習データ数 [分類] </summary>
    int trainNum[2];

    /* SOMアルゴリズム用変数 */
    /// <summary> ハードウェア呼出のパイプライン段数 </summary>
    const int pipeline_depth = 2;
    /// <summary> 選択した学習データ </summary>
    fixed16_2 *selectedVect[pipeline_depth];
    /// <summary> タイマー </summary>
    perf_counter ctr;

    /* selectedVectの領域を確保 */
    for (int i = 0; i < pipeline_depth; i++) {
        selectedVect[i] = (fixed16_2 *)sds_alloc(sizeof(fixed16_2) * HW_TIME * VECT_DIM);
    }
    /* vectMapの領域を確保 */
    fixed16_2 *base = (fixed16_2 *)sds_alloc(sizeof(fixed16_2) * MAP_SIZE * MAP_SIZE * VECT_DIM);
    for (int r = 0; r < MAP_SIZE; r++)
        for (int c = 0; c < MAP_SIZE; c++)
            vectMap[r][c] = base + (r * MAP_SIZE + c) * VECT_DIM;

    /* 学習データ読み込み */
    hangSampleData(trainData, trainNum);
    std::cout << "Loading complete. (Number of trainData(" << trainNum[0] << "," << trainNum[1] << "))" << std::endl;

    /* 初期化 */
    initVectMap(vectMap);
    som(vectMap[0][0], selectedVect[0], SOM_INIT);
    initLabelMap(labelMap);

     /* 学習前のマップに対しラベル付け */
    std::cout << std::endl;
    std::cout << "Labeling start : ";
    updateLabelMap(vectMap, labelMap, trainData, trainNum);
    std::cout << std::endl;
    std::cout << "Complete." << std::endl;

    /* マップの描画 */
    printf("***   INITIALIZED MAP (%d × %d, - : normal, o : defect)    ***\n", MAP_SIZE, MAP_SIZE);
    for (int y = 0; y < MAP_SIZE; y++) {
        for (int x = 0; x < MAP_SIZE; x++) {
            std::cout << std::setw(2) << (labelMap[y][x] ? "-" : "o");
        }
        std::cout << std::endl;
    }

    /* SOMアルゴリズム */
    std::cout << std::endl;
    std::cout << std::endl;
    std::cout << "Learning start : ";
    ctr.start();
    /* 学習データ選択と学習をパイプライン処理 */
    for (int i = 0; i < pipeline_depth; i++) {
        std::cout << "#";

        /* 学習データのベクトルを選択 */
        decideInputVect(trainData, trainNum, selectedVect[i]);

#pragma SDS async(1)
        som(vectMap[0][0], selectedVect[i], SOM_LEARN);
    }
    for (int i = pipeline_depth; i < LEARNING_TIME / HW_TIME; i++) {
        std::cout << "#";

        /* 学習データのベクトルを選択 */
        decideInputVect(trainData, trainNum, selectedVect[i % pipeline_depth]);

#pragma SDS wait(1)
#pragma SDS async(1)
        som(vectMap[0][0], selectedVect[i % pipeline_depth], SOM_LEARN);
    }
    for (int i = 0; i < pipeline_depth; i++) {
        std::cout << "#";
#pragma SDS wait(1)
    }
    ctr.stop();
    std::cout << std::endl;
    std::cout << "Complete." << std::endl;

    /* FPGA内部のBRAMからマップのベクトルデータを読む */
    som(vectMap[0][0], selectedVect[0], SOM_READ);

    std::cout << "Elapsed: " << ctr.avg_sec() << " [sec.]" << std::endl;
    std::cout << std::endl;
    std::cout << std::endl;

    /* ラベル付け */
    std::cout << "Labeling start : ";
    updateLabelMap(vectMap, labelMap, trainData, trainNum);
    std::cout << std::endl;
    std::cout << "Complete." << std::endl;

    /* マップの描画 */
    printf("***   LEARNED MAP (%d × %d, - : normal, o : defect)    ***\n", MAP_SIZE, MAP_SIZE);
    for (int y = 0; y < MAP_SIZE; y++) {
        for (int x = 0; x < MAP_SIZE; x++) {
            std::cout << std::setw(2) << (labelMap[y][x] ? "-" : "o");
        }
        std::cout << std::endl;
    }

    std::cout << std::endl;
    std::cout << "Finish." << std::endl;
}
