/*
 * sds_time.h
 *
 *  Created on: 2017/08/04
 *      Author: Komatsu
 */

#ifndef _SDS_TIME_H_
#define _SDS_TIME_H_

#include <stdint.h>
#include "sds_lib.h"
#define SDS_CLOCKS_PER_SEC 666666687

class perf_counter {
  public:
    uint64_t tot, cnt, calls;
    perf_counter() : tot(0), cnt(0), calls(0){};

    /**
     * メンバ変数のリセット
     */
    inline void reset() { tot = cnt = calls = 0; }

    /**
     * 計測スタート
     */
    inline void start() {
        cnt = sds_clock_counter();
        calls++;
    };

    /**
     * 計測ストップ
     */
    inline void stop() { tot += (sds_clock_counter() - cnt); };

    /**
     * 総クロック数を返す
     */
    inline uint64_t cpu_cycles() { return tot; };

    /**
     * 総処理時間を返す
     */
    inline double sec() { return (double)tot / SDS_CLOCKS_PER_SEC; }

    /**
     * 平均クロック数を返す
     */
    inline uint64_t avg_cpu_cycles() { return ((tot + (calls >> 1)) / calls); };

    /**
     * 平均処理時間を返す
     */
    inline double avg_sec() {
        return ((double)tot / calls / SDS_CLOCKS_PER_SEC);
    };
};

#endif /* _SDS_TIME_H_ */
