#include <cfloat>
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <iostream>

#include "env.h"
#include "som.h"
#include "tool.h"

void decideInputVect(fixed16_2 **data[2], int fileNum[2], fixed16_2 *selectedVect) {
    for (int t = 0; t < HW_TIME; t++) {
        //陰性、陽性をランダムで選択
        int cls = randN(2);
        //学習データの添え字からランダムでベクトルを選択
        int idx = randN(fileNum[cls]);

        memcpy(&selectedVect[t * VECT_DIM], data[cls][idx], sizeof(fixed16_2) * VECT_DIM);
    }
}

//ベクトルの距離計算
ufixed16_2 calcDist(fixed16_2 v1[VECT_DIM], fixed16_2 v2[VECT_DIM]) {
    ufixed16_2 retDist = 0.0;
    //それぞれの成分の差の絶対値の総和
    for (int i = 0; i < VECT_DIM; i++) {
        //差の絶対値
        ufixed16_2 diff = v1[i] > v2[i] ? v1[i] - v2[i] : v2[i] - v1[i];
        retDist += diff;
    }
    return retDist;
}

/// <summary>
/// データの中から最も近いものを探す
/// </summary>
/// <param name="target">探索対象</param>
/// <param name="data">全データ配列</param>
/// <param name="fileNum">全ファイル数</param>
/// <returns>最も近いデータの各次元の添字</returns>
int *searchNearestSample(fixed16_2 *target, fixed16_2 **data[2], int fileNum[2]) {
    ufixed16_2 min = 4.0;
    ufixed16_2 tmp = 0.0;
    int *sampleIdx = new int[2];

    // 最も近いデータを探索
    for (int i = 0; i < 2; i++) {
        for (int j = 0; j < fileNum[i]; j++) {
            tmp = calcDist(target, data[i][j]);
            if (tmp < min) {
                min = tmp;
                sampleIdx[0] = i;
                sampleIdx[1] = j;
            }
        }
    }

    return sampleIdx;
}

void updateLabelMap(fixed16_2 *vectMap[MAP_SIZE][MAP_SIZE], int labelMap[MAP_SIZE][MAP_SIZE], fixed16_2 **data[2],
                    int fileNum[2]) {
    int *sampleNum;

    for (int i = 0; i < MAP_SIZE; i++) {
        std::cout << "#";
        for (int j = 0; j < MAP_SIZE; j++) {
            // ニューロンに最も近いサンプルを検索
            sampleNum = searchNearestSample(vectMap[i][j], data, fileNum);

            // ラベルを更新
            labelMap[i][j] = sampleNum[0];

            delete[] sampleNum;
        }
    }
}
